@@ -7,7 +7,7 @@
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
-#   the Free Software Foundation, either version 3 of the License, or
+#   the Free Software Foundation, either version 2 of the License, or
 #   (at your option) any later version.
 #
 #   This program is distributed in the hope that it will be useful,
@@ -194,6 +194,9 @@ function(x) {
     if(any(indexClass(x)=='POSIXt')) {
       indexClass(x) <- "Date"  # set indexClass to Date
     }
+    if(any(indexClass(x) %in% .classesWithoutTZ)) {
+      indexTZ(x) <- "UTC"  # set indexTZ to UTC
+    }
     # force conversion, even if we didn't set indexClass to Date
     # because indexAt yearmon/yearqtr won't drop time from index
     index(x) <- index(x)
@@ -255,11 +258,13 @@ function(x, by, k=1, name=NULL, OHLC=TRUE, ...) {
     cnames <- NULL
 
   # start to.frequency-specific code
-  if (missing(by)) by <- 1:NROW(x)
+  if (missing(by)) by <- rep(1L, nrow(x))
   byVec <- cumsum(by)
   bins <- byVec %/% k
-  bins[1] <- 0
-  ep <- which(diff(bins)!=0)
+  # ep contents must have the same format as output generated by endpoints(): 
+  # first element must be zero and last must be nrow(x)
+  ep <- c(0L, which(diff(bins) != 0))
+  if (ep[length(ep)] != nrow(bins)) ep <- c(ep, nrow(bins))
   # end to.frequency-specific code
 
   xx <- .Call("toPeriod", 
